/***************************************************************/
/*                     bigrank.c                               */
/*　ngramの頻度分布を作成します(大容量版)                      */
/*　　　入力は標準入力から与え，出力は標準出力に出力します     */
/* 使い方　                                                    */
/*  $./rank < (入力ファイル名）> (出力ファイル名)              */
/*　入力ファイルには，ngramのファイルを指定します              */
/*　出力ファイルには，頻度分布を出力します                     */
/***************************************************************/

/* ヘッダファイルのインクルード*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*記号定数の定義*/
#define N 32 /*ngramの最大長*/
#define ITEMNO 65535*5 /*ngramの最大個数*/
#define NUMF 4 /*追加の数値フィールドの大きさ*/

/*関数のプロトタイプの宣言*/
int uniq(int n,char ngram[ITEMNO][N]
               ,char result[ITEMNO][N+NUMF]) ; /*重複をまとめる*/
int rescmp(const char *a,const char *b) ;/*比較関数*/

/*外部変数の定義*/
char ngram[ITEMNO][N] ; /*ngramなどを保持する変数*/
char result[ITEMNO][N+NUMF] ; /*ngramなどを保持する変数*/


/*******************/
/*    main()関数   */
/*******************/
int main(int argc,char *argv[])
{
 int i,j ; /*繰り返しの制御変数*/
 int no ; /*出力の個数*/

 /*ngramの読み込み*/
 i=0 ;
 while(fgets(ngram[i],N,stdin) != NULL){
  if(i>=ITEMNO){/*ngramが最大数に達した*/
   fprintf(stderr,"ngramが最大数に達しました\n") ;
   break ;
  }
  ++i ;
 }
 /*辞書順に整列*/
 qsort(ngram,i,N,
     (int (*)(const void *,const void *))strcmp) ; 
 /*重複をまとめる*/
 no=uniq(i,ngram,result) ;
 /*頻度順に整列*/
 qsort(result,no,N+NUMF,
     (int (*)(const void *,const void *))rescmp) ; 

 /*結果の出力*/
 for(j=0;j<i;++j) printf("%s",result[j]) ;
 
 return 0 ;
}

/*******************/
/*  uniq()関数     */
/* 重複をまとめる  */
/*******************/
int uniq(int n,char ngram[ITEMNO][N]
                ,char result[ITEMNO][N+NUMF])
{
 int i,j ;/*処理のカウンタ*/
 int c ;/*重複を数え上げる変数*/ 
 char lastgram[N] ;/*直前のngram*/ 

 /*初期化*/
 strncpy(lastgram,ngram[0],N) ;
 j=0 ;
 c=1 ;

 /*重複処理*/ 
 for(i=1;i<n;++i){
  if(strcmp(lastgram,ngram[i])==0){
   /*ngramが重複*/
   ++ c ;/*数え上げ*/
  }else{
   /*結果の配列に値を挿入*/
   sprintf(result[j],"%d\t%s",c,lastgram) ;
   strncpy(lastgram,ngram[i],N) ;
   c=1 ;
   ++j ;
  }
 }
 /*結果の配列に値を挿入*/
 sprintf(result[j],"%d\t%s",c,lastgram) ;
 strncpy(lastgram,ngram[i],N) ;
 ++j ;
 return j ; 
}
      
/*******************/
/* rescmp()関数    */
/* 比較関数        */
/*******************/
int rescmp(const char *a,const char *b)
{
 int numa,numb ;
 /*先頭の数字を数値に変換*/
 numa=atoi(a) ; numb=atoi(b) ;
 /*比較結果を戻り値とする*/
 if(numa>numb) return -1 ;
 else if(numa<numb) return 1 ;
 else return 0 ;
}




