/***************************************************************/
/*                       ngram.c                               */
/*　ngramを作成するプログラム                             　　 */
/*　　　入力は標準入力から与え，出力は標準出力に出力します     */
/* 使い方　                                                    */
/*  $./ngram 　(nの値) < (入力ファイル名）> (出力ファイル名)   */
/*　入力ファイルには，テキストファイルを指定します             */
/*　出力ファイルには，ngramを出力します                        */
/***************************************************************/

/* ヘッダファイルのインクルード*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*記号定数の定義*/
#define N 32 /*ngramの最大長*/

/*関数のプロトタイプの宣言*/
void  setlastch(int n,char chr,char lastch[]) ;
                           /*過去の文字列の保存*/
void  printngram(int n,char lastch[]) ;/*ngramの出力*/

/*******************/
/*    main()関数   */
/*******************/
int main(int argc,char *argv[])
{
 int n ; /*ngramの長さ*/
 char lastch[N]={' '} ; /*過去の値を保存する変数*/
 char chr ;/*入力された文字*/

 /*コマンド引数のチェック*/
 if(argc!=2){
  fprintf(stderr,"使い方　$./ngram (Nの値)"
                 " < (入力ファイル名）> (出力ファイル名)\n") ;
  exit(1) ;
 }

 /*Nの値の設定*/
 if(((n=atoi(argv[1]))<1)||(n>=N)){
  fprintf(stderr,"Nの値が不適切です\n") ;
  exit(1) ;
 }

 /*ngram作成の繰り返し*/
 while((chr=getchar())!=EOF){
  if(chr!='\n'){/*改行は除く*/
   /*過去の文字列の保存*/
   setlastch(n,chr,lastch) ;
   /*ngramの出力*/
   printngram(n,lastch) ;
  }
 }

 return 0 ;
}

/************************/
/*  setlastch()関数     */
/*  過去の文字列の保存  */
/************************/
void  setlastch(int n,char chr,char lastch[])
{
 int i ;

 /*過去のデータを順に送る*/
 for(i=n-2;i>=0;--i)
  lastch[i+1]=lastch[i] ;

 /*新しいデータの値をセット*/
 lastch[0]=chr ;
}

/************************/
/*  printngram()関数    */
/*  ngramの出力         */
/************************/
void  printngram(int n,char lastch[])
{
 int i ;
 
 /*ngramの出力*/
 for(i=n-1;i>=0;--i)
  printf("%c",lastch[i]) ;
 printf("\n") ;
}


