/***************************************************************/
/*                    sjisngram.c                              */
/*　sjisの全角文字で記述されたファイルについて，               */
/*       n-gramを作成するプログラム                            */
/*　　　入力は標準入力から与え，出力は標準出力に出力します     */
/* 使い方　                                                    */
/*  $./sjisngram (Nの値）< (入力ファイル名）> (出力ファイル名) */
/*　入力ファイルには，テキストファイルを指定します             */
/*　出力ファイルには，ngramを出力します                        */
/***************************************************************/

/* ヘッダファイルのインクルード*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*記号定数の定義*/
#define N 32 /*ngramの最大長*/

/*関数のプロトタイプの宣言*/
void  setlastch(int n,char chr,char lastch[]) ;
                           /*過去の文字列の保存*/
void  printngram(int n,char lastch[]) ;/*ngramの出力*/

/*******************/
/*    main()関数   */
/*******************/
int main(int argc,char *argv[])
{
 int i ; /*繰り返しの制御変数*/
 int n ; /*ngramの長さ*/
 char lastch[N]={' '} ; /*過去の値を保存する変数*/
 char chr ;/*入力された文字*/
 unsigned char d ;/*比較用*/

 /*コマンド引数のチェック*/
 if(argc!=2){
  fprintf(stderr,"使い方　$./sjisngram (Nの値)"
                 " < (入力ファイル名）> (出力ファイル名)\n") ;
  exit(1) ;
 }

 /*Nの値の設定*/
 if(((n=atoi(argv[1]))<1)||(n>=N/2)){
  fprintf(stderr,"Nの値が不適切です\n") ;
  exit(1) ;
 }

 /*sjisでは，2バイトで1文字を表現する*/
 n=n*2 ;

 /*ngram作成の繰り返し*/
 while((chr=getchar())!=EOF){
  d=(unsigned char) chr ;
  if(((d>0x7f)&&(d<0xa0))||(d>0xdf)&&(d<0xf0)){
                         /*sjisの2バイト文字*/
   /*過去の文字列の保存*/
   setlastch(n,chr,lastch) ;
   chr=getchar() ;
   setlastch(n,chr,lastch) ;

   /*ngramの出力*/
   printngram(n,lastch) ;
  }
 }

 return 0 ;
}

/************************/
/*  setlastch()関数     */
/*  過去の文字列の保存  */
/************************/
void  setlastch(int n,char chr,char lastch[])
{
 int i ;

 /*過去のデータを順に送る*/
 for(i=n-2;i>=0;--i)
  lastch[i+1]=lastch[i] ;

 /*新しいデータの値をセット*/
 lastch[0]=chr ;
}

/************************/
/*  printngram()関数    */
/*  ngramの出力         */
/************************/
void  printngram(int n,char lastch[])
{
 int i ;
 
 /*ngramの出力*/
 for(i=n-1;i>=0;--i)
  printf("%c",lastch[i]) ;
 printf("\n") ;
}


