/***************************************************************/
/*                     decision.c                              */
/*　判断木の構成を支援するプログラム　　                       */
/*　　　入力は標準入力から与え，出力は標準出力に出力します     */
/* 使い方　                                                    */
/*       $./decision < (入力ファイル名）> (出力ファイル名)     */
/*　入力ファイルでは，属性とカテゴリを0/1で記述します          */
/*　出力ファイルには，分類結果を表示します                     */
/***************************************************************/

/* ヘッダファイルのインクルード*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*記号定数の定義*/
#define TEXTLENGTH 256 /*テキストの最大長*/
#define DATANO 256   /*学習データセットの最大個数*/
#define ATTNO 4 /*属性値の個数*/
#define YES 1   /*yesに対応する整数値*/
#define NO 0    /*noに対応する整数値*/

/*関数のプロトタイプの宣言*/
int readdata(int dataset[DATANO][ATTNO+1]) ;
                     /*学習データ読み込み*/
void evalp(int dataset[DATANO][ATTNO+1],int no,int i,int yorn) ;
                /*     属性の評価       */

/*******************/
/*    main()関数   */
/*******************/
int main(int argc,char *argv[])
{
 int no ; /*学習データセットの個数*/
 int i ; /*繰り返しの制御変数*/
 int dataset[DATANO][ATTNO+1]={0} ;
       /*学習データセットの属性とカテゴリを格納*/

 /*学習データセットの読み込み*/
 no=readdata(dataset) ;

 /*各属性による分類*/
 for(i=0;i<ATTNO;++i){
  printf("\np%d\n",i+1) ;
  /*Yesに対応した学習データの数え上げ*/
  printf("Yes ") ;
  evalp(dataset,no,i,YES) ;
  /*Noに対応した学習データの数え上げ*/
  printf("No  ") ;
  evalp(dataset,no,i,NO) ;
 }
 
 return 0 ;
}

/************************/
/*   readdata()関数     */
/* 学習データの読み込み */
/************************/
int readdata(int dataset[DATANO][ATTNO+1])
{
 int no=0 ; /*学習データセットの個数*/
 int att=0 ; /*学習データセットの属性*/

 /*データの入力*/
 while(scanf("%d",&dataset[no][att])!=EOF){
  ++ att ;
  if(att>ATTNO){/*次のデータ*/
   att=0 ;
   ++no ;
  }
 }
 return no ;
}



/************************/
/*     evalp()関数      */
/*     属性の評価       */
/************************/
void evalp(int dataset[DATANO][ATTNO+1],int no,int i,int yorn)
{
 int j ; /*繰り返しの制御変数*/
 int total,point;/*評価値*/

 /*yornに対応した学習データの数え上げ*/
 j=0 ;
 total=point=0 ;
 while(j<no){/*各データについて調べる*/
  if(dataset[j][i]==yorn){
   ++ total ;/*該当項目*/
   if(dataset[j][i]==dataset[j][ATTNO])
    ++ point ;/*分類できた*/
  }
  ++j ;
 }
 /*結果の出力*/ 
 printf("%d/%d  %lf\n",point,total,(double)point/total) ;
}

