/*****************************************************/
/*                     gaml.c                        */
/*　SGAに基づく知識獲得プログラム                    */
/* 使い方　                                          */
/*   $./gaml < (入力ファイル名）                     */
/*入力ファイルでは，属性とカテゴリを0/1で記述します  */
/*****************************************************/

/* ヘッダファイルのインクルード*/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

/*記号定数の定義*/
#define POOLSIZE 30 /*プールサイズ*/
#define LASTG 200    /*打ち切り世代*/
#define MRATE 0.01  /*突然変異の確率*/
#define SEED 65535    /*乱数のシード*/
#define YES 1   /*yesに対応する整数値*/
#define NO 0    /*noに対応する整数値*/
#define TEXTLENGTH 256 /*テキストの最大長*/
#define DATANO 256   /*学習データセットの最大個数*/
#define ATTNO 4 /*属性値の個数*/
#define TERMNO 3 /*論理式の論理和項の個数*/

/*関数のプロトタイプの宣言*/
int evalfit(int gene[]) ; /*適応度の計算*/
void mating(int pool[POOLSIZE][TERMNO*ATTNO*2]) ; /*交叉*/
void mutation(int pool[POOLSIZE][TERMNO*ATTNO*2]) ; /*突然変異*/
void printp(int pool[POOLSIZE][TERMNO*ATTNO*2]) ; /*結果出力*/
void initpool(int pool[POOLSIZE][TERMNO*ATTNO*2]) ;
                         /*初期集団の生成*/
int rndn() ; /*n未満の乱数の生成   */
int notval(int v) ;/*真理値の反転*/
int select(int roulette[POOLSIZE],int totalfitness) ;
                    /*選択*/
void copypool(int pool[POOLSIZE][TERMNO*ATTNO*2]
          ,int nextpool[POOLSIZE][TERMNO*ATTNO*2]) ;
                 /*次世代のプールをコピー*/
void crossing(int m[],int p[],int c1[],int c2[]) ;
                /* 特定の２遺伝子の交叉 */
int readdata(int dataset[DATANO][ATTNO+1]) ;
                     /*学習データ読み込み*/
int evals(int formula[TERMNO*ATTNO*2]
      ,int subset[ATTNO+1]); /*特定の事例に対する評価*/

/*問題の数値を格納する配列*/
int no ; /*学習データセットの個数*/
int dataset[DATANO][ATTNO+1]={0} ;
       /*学習データセットの属性とカテゴリを格納*/


/*******************/
/*    main()関数   */
/*******************/
int main(int argc,char *argv[])
{
 int pool[POOLSIZE][TERMNO*ATTNO*2] ; /*遺伝子プール*/
 int generation;/* 現在の世代数 */

 /*乱数の初期化*/
 srand(SEED) ;

 /*学習データセットの読み込み*/
 no=readdata(dataset) ;

 /*初期集団の生成*/
 initpool(pool) ;

 /*打ち切り世代まで繰り返し*/
 for(generation=0;generation<LASTG;++generation){
  printf("%d世代\n",generation) ;
  mating(pool) ;/*交叉*/
  mutation(pool) ;/*突然変異*/
  printp(pool) ;/*結果出力*/
 }
 
 return 0 ;
}


/************************/
/*   select()関数       */
/*       選択           */
/************************/
int select(int roulette[POOLSIZE],int totalfitness)
{
 int i ;/* 繰り返しの制御変数 */
 int ball ;/* 玉（選択位置の数値）*/
 int acc=0 ;/*適応度の積算値*/

 ball=rndn(totalfitness) ;
 for(i=0;i<POOLSIZE;++i){
  acc+=roulette[i] ;/*適応度を積算*/
  if(acc>ball) break ;/*対応する遺伝子*/
 }
   
 return i ;

}


/************************/
/*   mating()関数       */
/*        交叉          */
/************************/
void mating(int pool[POOLSIZE][TERMNO*ATTNO*2])
{
 int i ;/* 繰り返しの制御変数 */
 int totalfitness=0 ;/*適応度の合計値*/
 int nextpool[POOLSIZE][TERMNO*ATTNO*2] ;/*子の世代*/
 int roulette[POOLSIZE] ;/*適応度を格納*/
 int mama,papa ;/*親の遺伝子の番号*/

 /* ルーレットの作成 */
 for(i=0;i<POOLSIZE;++i){
  roulette[i]=evalfit(pool[i]) ;
  /* 適応度の合計値を計算*/
  totalfitness+=roulette[i] ;
 }

 /*選択と交叉を繰り返す*/
 for(i=0;i<POOLSIZE/2;++i){
  do{/*親の選択*/
   mama=select(roulette,totalfitness) ;
   papa=select(roulette,totalfitness) ;
  }while(mama==papa) ;/*重複の削除*/
  /*特定の２遺伝子の交叉*/
  crossing(pool[mama],pool[papa]
            ,nextpool[i*2],nextpool[i*2+1])  ;
 }

 /*次世代のプールをコピー*/
 copypool(pool,nextpool) ;

}
/************************/
/*  crossing()関数      */
/* 特定の２遺伝子の交叉 */
/************************/
void crossing(int m[],int p[],int c1[],int c2[])
{
 int j ;/* 繰り返しの制御変数 */
 int cp ;/*交叉する点*/

 /*交叉点の決定*/
 cp =rndn(TERMNO*ATTNO*2) ;

 /*遺伝子の前半部分のコピー*/
 for(j=0;j<cp;++j){
  c1[j]=m[j] ;
  c2[j]=p[j] ;
 }
 
 /*遺伝子の後半部分のコピー*/
 for(;j<TERMNO*ATTNO*2;++j){
  c2[j]=m[j] ;
  c1[j]=p[j] ;
 }
}

/************************/
/*  copypool()関数      */
/*次世代のプールをコピー*/
/************************/
void copypool(int pool[POOLSIZE][TERMNO*ATTNO*2]
          ,int nextpool[POOLSIZE][TERMNO*ATTNO*2])
{
 int i,j ;/* 繰り返しの制御変数 */

 for(i=0;i<POOLSIZE;++i)
  for(j=0;j<TERMNO*ATTNO*2;++j)
   pool[i][j]=nextpool[i][j] ;
}

/************************/
/*   evalfit()関数      */
/*   適応度の計算       */
/************************/
int evalfit(int g[])
{
 int i ;/*ループ制御の変数*/
 int point=0 ;/*正解の個数*/

 /*学習セットの各事例に論理式を適用*/
 for(i=0;i<no;++i){
  point+=evals(g,dataset[i]) ;
 }

 return point*100/no ;/*評価値を戻す*/
}

/************************/
/*    evals()関数       */
/*特定の事例に対する評価*/
/************************/
int evals(int formula[TERMNO*ATTNO*2]
  ,int subset[ATTNO+1])
{
 int orval,andval,result ;/*計算の途中経過値*/
 int i,j ;/*ループ制御の変数*/

 andval=YES ;
 /*論理積で項を結んだ，式全体の評価*/
 for(i=0;i<TERMNO;++i){
  orval=NO ;
  /*論理和で結んだ項の評価*/
  for(j=0;j<ATTNO*2;++j){
   if((j%2)==0) orval+=subset[j/2]*formula[i*ATTNO*2+j] ;
   else orval+=notval(subset[j/2])*formula[i*ATTNO*2+j] ;
  }
  if(orval==NO) andval=NO ;
  else andval*=YES ;
 }

 if(andval==subset[ATTNO]) return 1 ;/*結果が一致*/
 else return 0 ;/*不一致*/
}

/***********************/
/*   printp()関数      */
/*   結果出力          */
/***********************/
void printp(int pool[POOLSIZE][TERMNO*ATTNO*2])
{
 int i,j ;/* 繰り返しの制御変数 */
 int fitness ;/* 適応度 */
 double totalfitness=0 ;/* 適応度の合計値 */
 int elite,bestfit=0 ;/*エリート遺伝子の処理用変数*/ 

 for(i=0;i<POOLSIZE;++i){
  for(j=0;j<TERMNO*ATTNO*2;++j)
   printf("%1d",pool[i][j]) ;
  fitness=evalfit(pool[i]) ;
  printf("\t%d\n",fitness) ;
  if(fitness>bestfit){/*エリート解*/
   bestfit=fitness ;
   elite=i ;
  }
  totalfitness+=fitness ;
 }
 /*エリート解の適応度を出力*/
 printf("%d\t%d   ",elite,bestfit) ;
 /*平均の適応度を出力*/
 printf("%lf\n",totalfitness/POOLSIZE) ;
}


/***********************/
/*   initpool()関数    */
/*   初期集団の生成    */
/***********************/
void initpool(int pool[POOLSIZE][TERMNO*ATTNO*2])
{
 int i,j ;/* 繰り返しの制御変数 */

 for(i=0;i<POOLSIZE;++i)
  for(j=0;j<TERMNO*ATTNO*2;++j)
   pool[i][j]=rndn(2) ;

}


/************************/
/*     rndn()関数       */
/*    n未満の乱数の生成 */
/************************/
int rndn(int l)
{
 int rndno ;/*生成した乱数*/

 while((rndno=((double)rand()/RAND_MAX)*l)==l) ;

 return rndno;
}

/***********************/
/*   mutation()関数    */
/*   突然変異          */
/***********************/
void mutation(int pool[POOLSIZE][TERMNO*ATTNO*2])
{
 int i,j ;/* 繰り返しの制御変数 */

 for(i=0;i<POOLSIZE;++i)
  for(j=0;j<TERMNO*ATTNO*2;++j)
   if((double)rndn(100)/100.0<=MRATE)
    /*反転の突然変異*/
    pool[i][j]=notval(pool[i][j]) ;

}


/************************/
/*   notval()関数       */
/*   真理値の反転       */
/************************/
int notval(int v)
{
 if(v==YES) return NO ;
 else return YES ;
}

/************************/
/*   readdata()関数     */
/* 学習データの読み込み */
/************************/
int readdata(int dataset[DATANO][ATTNO+1])
{
 int no=0 ; /*学習データセットの個数*/
 int att=0 ; /*学習データセットの属性*/

 /*データの入力*/
 while(scanf("%d",&dataset[no][att])!=EOF){
  ++ att ;
  if(att>ATTNO){/*次のデータ*/
   att=0 ;
   ++no ;
  }
 }
 return no ;
}
