/*****************************************************/
/*                       sga.c                       */
/*　SGAの説明用プログラム　    　                    */
/* 使い方　                                          */
/*       $./sga                                      */
/*****************************************************/

/* ヘッダファイルのインクルード*/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

/*記号定数の定義*/
#define DATANO 20   /*データ数*/
#define POOLSIZE 30 /*プールサイズ*/
#define LASTG 200    /*打ち切り世代*/
#define MRATE 0.01  /*突然変異の確率*/
#define SEED 65535    /*乱数のシード*/
#define YES 1   /*yesに対応する整数値*/
#define NO 0    /*noに対応する整数値*/
#define VALUE 499 /*目標値*/
#define MAXFIT 1000    /*適応度の計算に用いる数値*/

/*関数のプロトタイプの宣言*/
int evalfit(int gene[]) ; /*適応度の計算*/
void mating(int pool[POOLSIZE][DATANO]) ; /*交叉*/
void mutation(int pool[POOLSIZE][DATANO]) ; /*突然変異*/
void printp(int pool[POOLSIZE][DATANO]) ; /*結果出力*/
void initpool(int pool[POOLSIZE][DATANO]) ;
                         /*初期集団の生成*/
int rndn() ; /*n未満の乱数の生成   */
int notval(int v) ;/*真理値の反転*/
int select(int roulette[POOLSIZE],int totalfitness) ;
                    /*選択*/
void copypool(int pool[POOLSIZE][DATANO]
          ,int nextpool[POOLSIZE][DATANO]) ;
                 /*次世代のプールをコピー*/
void crossing(int m[],int p[],int c1[],int c2[]) ;
                /* 特定の２遺伝子の交叉 */

/* 問題の数値の設定 */
int q[]={31,41,59,26,53,58,97,93,23,84,
         -62,-64,-33,-83,-27,-95,-2,-88,-41,-97} ;

/*******************/
/*    main()関数   */
/*******************/
int main(int argc,char *argv[])
{
 int pool[POOLSIZE][DATANO] ; /*遺伝子プール*/
 int generation;/* 現在の世代数 */

 /*乱数の初期化*/
 srand(SEED) ;

 /*初期集団の生成*/
 initpool(pool) ;

 /*打ち切り世代まで繰り返し*/
 for(generation=0;generation<LASTG;++generation){
  printf("%d世代\n",generation) ;
  mating(pool) ;/*交叉*/
  mutation(pool) ;/*突然変異*/
  printp(pool) ;/*結果出力*/
 }
 
 return 0 ;
}


/************************/
/*   select()関数       */
/*       選択           */
/************************/
int select(int roulette[POOLSIZE],int totalfitness)
{
 int i ;/* 繰り返しの制御変数 */
 int ball ;/* 玉（選択位置の数値）*/
 int acc=0 ;/*適応度の積算値*/

 ball=rndn(totalfitness) ;
 for(i=0;i<POOLSIZE;++i){
  acc+=roulette[i] ;/*適応度を積算*/
  if(acc>ball) break ;/*対応する遺伝子*/
 }
   
 return i ;

}


/************************/
/*   mating()関数       */
/*        交叉          */
/************************/
void mating(int pool[POOLSIZE][DATANO])
{
 int i ;/* 繰り返しの制御変数 */
 int totalfitness=0 ;/*適応度の合計値*/
 int nextpool[POOLSIZE][DATANO] ;/*子の世代*/
 int roulette[POOLSIZE] ;/*適応度を格納*/
 int mama,papa ;/*親の遺伝子の番号*/

 /* ルーレットの作成 */
 for(i=0;i<POOLSIZE;++i){
  roulette[i]=evalfit(pool[i]) ;
  /* 適応度の合計値を計算*/
  totalfitness+=roulette[i] ;
 }

 /*選択と交叉を繰り返す*/
 for(i=0;i<POOLSIZE/2;++i){
  do{/*親の選択*/
   mama=select(roulette,totalfitness) ;
   papa=select(roulette,totalfitness) ;
  }while(mama==papa) ;/*重複の削除*/
  /*特定の２遺伝子の交叉*/
  crossing(pool[mama],pool[papa]
            ,nextpool[i*2],nextpool[i*2+1])  ;
 }

 /*次世代のプールをコピー*/
 copypool(pool,nextpool) ;

}
/************************/
/*  crossing()関数      */
/* 特定の２遺伝子の交叉 */
/************************/
void crossing(int m[],int p[],int c1[],int c2[])
{
 int j ;/* 繰り返しの制御変数 */
 int cp ;/*交叉する点*/

 /*交叉点の決定*/
 cp =rndn(DATANO) ;

 /*遺伝子の前半部分のコピー*/
 for(j=0;j<cp;++j){
  c1[j]=m[j] ;
  c2[j]=p[j] ;
 }
 
 /*遺伝子の後半部分のコピー*/
 for(;j<DATANO;++j){
  c2[j]=m[j] ;
  c1[j]=p[j] ;
 }
}

/************************/
/*  copypool()関数      */
/*次世代のプールをコピー*/
/************************/
void copypool(int pool[POOLSIZE][DATANO]
          ,int nextpool[POOLSIZE][DATANO])
{
 int i,j ;/* 繰り返しの制御変数 */

 for(i=0;i<POOLSIZE;++i)
  for(j=0;j<DATANO;++j)
   pool[i][j]=nextpool[i][j] ;
}

/************************/
/*   evalfit()関数      */
/*   適応度の計算       */
/************************/
int evalfit(int g[])
{
 int i ;/* 繰り返しの制御変数 */
 int fitness=0 ;/*適応度の途中値*/

 for(i=0;i<DATANO;++i)
  fitness+=g[i]*q[i] ;

 return MAXFIT-abs(VALUE-fitness) ;
}

/***********************/
/*   printp()関数      */
/*   結果出力          */
/***********************/
void printp(int pool[POOLSIZE][DATANO])
{
 int i,j ;/* 繰り返しの制御変数 */
 int fitness ;/* 適応度 */
 double totalfitness=0 ;/* 適応度の合計値 */
 int elite,bestfit=0 ;/*エリート遺伝子の処理用変数*/ 

 for(i=0;i<POOLSIZE;++i){
  for(j=0;j<DATANO;++j)
   printf("%1d",pool[i][j]) ;
  fitness=evalfit(pool[i]) ;
  printf("\t%d\n",fitness) ;
  if(fitness>bestfit){/*エリート解*/
   bestfit=fitness ;
   elite=i ;
  }
  totalfitness+=fitness ;
 }
 /*エリート解の適応度を出力*/
 printf("%d\t%d   ",elite,bestfit) ;
 /*平均の適応度を出力*/
 printf("%lf\n",totalfitness/POOLSIZE) ;
}


/***********************/
/*   initpool()関数    */
/*   初期集団の生成    */
/***********************/
void initpool(int pool[POOLSIZE][DATANO])
{
 int i,j ;/* 繰り返しの制御変数 */

 for(i=0;i<POOLSIZE;++i)
  for(j=0;j<DATANO;++j)
   pool[i][j]=rndn(2) ;

}


/************************/
/*     rndn()関数       */
/*    n未満の乱数の生成 */
/************************/
int rndn(int l)
{
 int rndno ;/*生成した乱数*/

 while((rndno=((double)rand()/RAND_MAX)*l)==l) ;

 return rndno;
}

/***********************/
/*   mutation()関数    */
/*   突然変異          */
/***********************/
void mutation(int pool[POOLSIZE][DATANO])
{
 int i,j ;/* 繰り返しの制御変数 */

 for(i=0;i<POOLSIZE;++i)
  for(j=0;j<DATANO;++j)
   if((double)rndn(100)/100.0<=MRATE)
    /*反転の突然変異*/
    pool[i][j]=notval(pool[i][j]) ;

}


/************************/
/*   notval()関数       */
/*   真理値の反転       */
/************************/
int notval(int v)
{
 if(v==YES) return NO ;
 else return YES ;
}

