/********************************************************/
/*                       puzzle.c                       */
/*　パズルの解を求めるプログラム　　                    */
/* 使い方　                                             */
/*       $./puzzle                                      */
/********************************************************/

/* ヘッダファイルのインクルード*/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

/*記号定数の定義*/
#define DATANO 20   /*データ数*/
#define VALUE 499   /*目標値*/

/*関数のプロトタイプの宣言*/
int evalfit(int g[]) ; /*適応度の計算*/
int inc(int g[]) ; /*制御配列のインクリメント*/
void printg(int g[],int fitness) ; /*結果出力*/

/* 問題の数値の設定 */
int q[]={31,41,59,26,53,58,97,93,23,84,
         -62,-64,-33,-83,-27,-95,-2,-88,-41,-97} ;

/*******************/
/*    main()関数   */
/*******************/
int main(int argc,char *argv[])
{
 int g[DATANO]={0} ; /*繰り返しの制御変数*/
 int bestfitness=INT_MAX ; /* 適応度の最適値 */
 int fitness ;/* 現在の適応度 */

 /* すべての場合の組み合わせを生成*/
 while(inc(g)==0){/*最後の桁上がりがあるまで繰り返し*/
  fitness=evalfit(g) ;
  if(abs(fitness)<=abs(bestfitness)){
   bestfitness=fitness ;/* 更新 */
   printg(g,fitness) ;/*結果出力*/
  }
 }
 
 return 0 ;
}

/************************/
/*   evalfit()関数      */
/*   適応度の計算       */
/************************/
int evalfit(int g[])
{
 int i ;/* 繰り返しの制御変数 */
 int fitness=0 ;/*適応度の途中値*/

 for(i=0;i<DATANO;++i)
  fitness+=g[i]*q[i] ;

 return VALUE-fitness ;
}

/***********************/
/*   inc()関数         */
/*  g[]のインクリメント*/
/***********************/
int inc(int g[])
{
 int i ;/* 繰り返しの制御変数 */
 int carry=1 ;/* 桁上がりの値 */

 for(i=0;i<DATANO;++i){
  g[i]+=carry ;
  carry=0 ;
  if(g[i]>1){
   g[i]=0 ;
   carry=1 ;
  }
 } 

 return carry;
}

/***********************/
/*   printg()関数      */
/*   結果出力          */
/***********************/
void printg(int g[],int fitness)
{
 int i ;/* 繰り返しの制御変数 */

 for(i=0;i<DATANO;++i)
  printf("%1d",g[i]) ;
 printf("\t%d\n",fitness) ;
}

